const global_uuid = `FACD900A-9868-4CCB-B660-9DDF52D1BE12`;

const symbol = Symbol.for(
	`@anio-js-foundation/anio-global-store-${global_uuid}`
);

//
// return the global store
// create it if it doesn't exist yet
//
function getGlobalStore() {
	if (symbol in globalThis) {
		return globalThis[symbol]
	}

	globalThis[symbol] = {
		//
		// private_items will never be
		// passed down to a worker created 
		// by @anio-js-foundation/node-create-worker
		//
		private_items: new Map(),
		//
		// shared_items will be passed
		// down to a worker created
		// by @anio-js-foundation/node-create-worker
		//
		shared_items: new Map()
	};

	return globalThis[symbol]
}

function genericSetItem(map, key, value) {
	const previous_item_value = map.has(key) ? map.get(key) : null;

	map.set(key, value);

	return previous_item_value
}

function genericGetItem(map, key) {
	if (!map.has(key)) {
		throw new Error(`No such key '${key}' in anio-global-store. (operation: get)`)
	}

	return map.get(key)
}

function genericDeleteItem(map, key) {
	if (!map.has(key)) {
		throw new Error(`No such key '${key}' in anio-global-store. (operation: delete)`)
	}

	map.delete(key);
}



//
// private globals
//
function hasItem(key) {
	const {private_items} = getGlobalStore();

	return private_items.has(key)
}

function setItem(key, value) {
	const {private_items} = getGlobalStore();

	return genericSetItem(private_items, key, value)
}

function getItem(key) {
	const {private_items} = getGlobalStore();

	return genericGetItem(private_items, key)
}

function deleteItem(key) {
	const {private_items} = getGlobalStore();

	genericDeleteItem(private_items, key);
}

function getItems() {
	const {private_items} = getGlobalStore();

	// only return a copy
	return new Map(private_items)
}

//
// "shared" globals
//
function hasSharedItem(key) {
	const {shared_items} = getGlobalStore();

	return shared_items.has(key)
}

function setSharedItem(key, value) {
	const {shared_items} = getGlobalStore();

	return genericSetItem(shared_items, key, value)
}

function getSharedItem(key) {
	const {shared_items} = getGlobalStore();

	return genericGetItem(shared_items, key)
}

function deleteSharedItem(key) {
	const {shared_items} = getGlobalStore();

	genericDeleteItem(shared_items, key);
}

function getSharedItems() {
	const {shared_items} = getGlobalStore();

	return new Map(shared_items)
}

//
var index = {
	hasItem,
	setItem,
	getItem,
	deleteItem,
	getItems,

	hasSharedItem,
	setSharedItem,
	getSharedItem,
	deleteSharedItem,
	getSharedItems
};

export { index as default, deleteItem, deleteSharedItem, getItem, getItems, getSharedItem, getSharedItems, hasItem, hasSharedItem, setItem, setSharedItem };
